% LQ tracking problem in Matlab
%
% (c) Hans Amman and David Kendrick
% Computing the Steady State of Linear Quadratic 
% with Rational Expectations
%
% Using QZ method for generalized eigenvalues 
% to solve RE part of the model.
%
%
% Example: 
%          x(t)  = c(t) + i(t)  + g(t)                                  
%          c(t)  = 0.80(x(t-1)-tax(t-1))+ 200                           
%          i(t)  = 0.20(Ex(t+1))+100                                    
%          g(t)  = u(t-1)                                               
%          tax(t)= 0.25*x(t)                                            
%                                                                       
%   min    J = 1/2 (sum(x(t)-1600)^2+sum(g(t))^2)                       
%
%
% Basic structure is:
%
% | 1  -0.2 | | x(t+1) |  = |0.6  0 | |  x(t) |  + u(t) + 300   
% | 1    0  | |Ex(t+2) |    | 0   1 | | x(t+1)|  
%

%... echo on; %<---<<<

clear;

n = 2; 
m = 1; 
T = 25; 
s = 100; 

adjust  = 0.5; 
eps     = 1.0e-09; 
maxiter = 1000;

A  =  [0.6];

B  =  [1];

C  =  [300];

D2 = [0.2];

x0 = [1500];

zt   = ones(1,T+s);

xtar = [1600]; %... [1600];

utar = [12];    %... [12];  

W    = [1.0];

% matrix for conditions on terminal states 

WT   = [1.0];

F    = [0];

R    = [1.0];

% Setting up augmented system xhat

G0 = zeros(2,2);
G0(1,1) =  1.0;
G0(1,2) = -D2;				
G0(2,1) =  1.0;
G0(2,2) =  0.0;


G1 = zeros(2,2);
G1(1,1)= A;
G1(1,2)= 0.0;
G1(2,1)= 0.0;
G1(2,2)= 1.0;				

G2=zeros(2,1);
G2(1,1)= B;

G3     = zeros(2,1);
G3(1,1)= C; 

% make qz decomposition
[OMEGA, LAMBDA, Q, Z, V] = qz(G1, G0);

% contains the eigenvalues of the generalized problem
eigen_values = diag(OMEGA) ./ diag(LAMBDA);


LAMBDA11   = LAMBDA(1,1);
LAMBDA12   = LAMBDA(1,2);
LAMBDA22   = LAMBDA(2,2);

OMEGA11    = OMEGA(1,1);
OMEGA12    = OMEGA(1,2);
OMEGA22    = OMEGA(2,2);

Q1         = Q(1,1:2);
Q2         = Q(2,1:2);

M          = inv(OMEGA22)*LAMBDA22;

LAMBDA_HAT       = zeros(2,2);
LAMBDA_HAT(1,1)  = LAMBDA11;
LAMBDA_HAT(1,2)  = LAMBDA12;
LAMBDA_HAT(2,2)  = 1.0;

Q1HAT   = zeros(2,2);
Q1HAT(1,1:2) = Q1;

OMEGA_HAT = zeros(2,2);
OMEGA_HAT(1,1)   = OMEGA11;
OMEGA_HAT(1,2)   = OMEGA12;

G2HAT = zeros(2,1);
G2HAT = Q1HAT*G2;

G3HAT = zeros(2,1);
G3HAT = Q1HAT*G3;

LAMBDA_HAT_INV = inv(LAMBDA_HAT);

AHAT = zeros(2,2);
AHAT = Z*LAMBDA_HAT_INV*OMEGA_HAT*Z';

BHAT = zeros(2,2);
BHAT = Z*LAMBDA_HAT_INV*G2HAT;

CHAT = zeros(2,3);
CHAT(1:2,1)  = Z*LAMBDA_HAT_INV*G3HAT;
CHAT(1:2,2:3)= Z*LAMBDA_HAT_INV;

x0_hat = zeros(2,1);
x0_hat(1,1) = x0(1,1);

xtar_hat = zeros(2,1);
xtar_hat(1,1) = xtar(1,1);

W_HAT = zeros(2,2);
W_HAT(1,1) = W(1,1);

WT_HAT = zeros(2,2);
WT_HAT(1,1) = WT(1,1);

F_HAT = zeros(2,1);
F_HAT(1,1) = F(1,1);

%Initial values of controls are set to steady state values
uiter =  19.23*ones(m,T+s);

uold  =  zeros(m,T);
w2    =  zeros(1,T);
ct    =  zeros(2,T);
zthat =  zeros(3,T);


for iter=1:maxiter,       disp(['  iter = ',num2str(iter),]) 

	
%Make forward projection with unstable root
% w2(1), w2(2), ....   are computed here

for k=1:T,  
    	j=0;
	sumw2=0.0;
	for i=k+1:T+s,
		j=j+1;
		tolz = (M^(j-1))*inv(OMEGA22)*Q2*(G2*uiter(1,i)+G3*zt(1,i));
		sumw2=sumw2+tolz;
	end;
	w2(1,k)=-sumw2;
end;			

for k=1:T,
	zthat(1,k) = zt(1,k);
	zthat(3,k) = w2(1,k);
end;	


% set begin value of augmented system

	x0_hat = zeros(2,1);
	x0_hat(1,1) = x0(1,1);
	
	if iter==1;
		x0_hat(2,1) = x0(1,1); 
	else;
		x0_hat(2,1) = x_hat(1,2);
	end;
						            % start Ricatti loop 

	u =  zeros(m,T);
	x_hat =  zeros(n,T+1);

	KOLD=WT_HAT;                   	% boundery condition 
	POLD=-WT_HAT*xtar_hat; 			% boundery condition 

	KLARGE=zeros(n*n,T);
	PLARGE=zeros(n,T);

	KLARGE(1:n*n,T) = reshape(KOLD(1:n,1:n),n*n,1);
	PLARGE(1:n,T)   = POLD(1:n);

	k=T-1;
	while k>=1,

		chat   = CHAT*zthat(1:3,k+1);
		WSMALL = -W_HAT*xtar_hat;
		RSMALL = -R*utar;

			% computing the ricatti matrices %

		KNEW=AHAT'*KOLD*AHAT+W_HAT-(AHAT'*KOLD*BHAT+F)* ...
	          inv(BHAT'*KOLD*BHAT+R')*(F_HAT'+BHAT'*KOLD*AHAT);

			% computing the tracking equation %

		PNEW=-(AHAT'*KOLD*BHAT+F_HAT)*inv(BHAT'*KOLD*BHAT+R')* ...
                   (BHAT'*(KOLD*chat+POLD)+RSMALL)+ ...
                    AHAT'*(KOLD*chat+POLD)+WSMALL;

			% setup next period %
	
		KOLD=KNEW;
        	POLD=PNEW;

		KTEMP(1:n*n,1)  = reshape(KNEW,n*n,1);
		KLARGE(1:n*n,k) = KTEMP(1:n*n,1);
		PLARGE(1:n,k)   = PNEW(1:n);
		
	        k=k-1;
	end;					    % end of the Ricatti loop %

	k=0;
	xold_hat = x0_hat;
	sumtmp   = zeros(1,1);

	while  k <= T-1,			% start of forward loop

		chat   = CHAT*zthat(1:3,k+1);			
		WSMALL = -W*xtar_hat;
		RSMALL = -R*utar;

		KTEMP(1:n*n,1) = KLARGE(1:n*n,k+1);
                  KOLD = reshape(KTEMP(1:n*n),n,n);
		          POLD =  PLARGE(1:n,k+1);

		GLARGE = -inv(BHAT'*KOLD*BHAT+R')*(F_HAT'+BHAT'*KOLD*AHAT);
    	GSMALL = -inv(BHAT'*KOLD*BHAT+R')*(BHAT'*(KOLD*chat+POLD)+RSMALL);

		uopt     = GLARGE*xold_hat + GSMALL;
    	xnew_hat = AHAT*xold_hat+BHAT*uopt+chat;

		x_hat(1:n,k+1) = xold_hat;
            u(1:m,k+1) = uopt;
		k=k+1;

           	xold_hat = xnew_hat;

	end;				% end of forward loop

	x_hat(1:n,T+1) = xold_hat;
	uiter(1,1:T) = adjust*u(1,1:T)+(1-adjust)*uiter(1,1:T);

						% fix boundery condition

% Boundery conditions are set to the steady state values

	for j=T+1:T+s,
		uiter(1,j) = adjust*u(1,T)+(1-adjust)*uiter(1,j);
		uiter(1,j) = 19.23;
	end;

	div  = reshape(u-uold,T*m,1);
	eps1 = div'*div;

	disp('   Value of eps1 = '),disp(eps1) 

	for k=1:T,		
	   sumtmp = sumtmp + ...
               0.5*(x_hat(1,k)-xtar_hat)'*W_HAT*(x_hat(1,k)-xtar_hat) + ... 
	           0.5*(u(1,k)-utar)'*R*(u(1,k)-utar) + ...
		       (x_hat(1,k)-xtar)'*F_HAT*(u(1,k)-utar);
	end;

	sumtmp = sumtmp + ...
		    0.5*(x_hat(1,T+1)-xtar_hat)'*WT_HAT*(x_hat(1,T+1)-xtar_hat);
format bank	
	disp('   Value J = '),disp(sumtmp)
format

	if eps1<eps,
		break;
	end;	

	uold=u;

end;				% end of iter loop

					% post iteration evaluation

					% Compute originl states		
x(1,1) = x0;
for k=1:T,
	x(1,k+1) = A*x(1,k)+B*u(1,k)+C*zt(1,k)+D2*x_hat(2,k+1);
end;

sum=0.0;	
for k=1:T,		
	sum = sum + ...
              0.5*(x(1,k)-xtar)'*W*(x(1,k)-xtar) + ... 
	          0.5*(u(1,k)-utar)'*R*(u(1,k)-utar) + ...
		      (x(1,k)-xtar)'*F*(u(1,k)-utar);
end;

	sum = sum + 0.5*(x(1,T+1)-xtar)'*WT*(x(1,T+1)-xtar);
    
%...................			
t = (0:length(u'))';
%...................

%Output of relevant result

format bank

disp('         The target control/state values  ')
disp(['         u_tar = ',num2str(xtar),])
disp(['         x_tar = ',num2str(utar),])
disp(' ');

disp('         The optimal control/state vector ')
disp(['         x_opt(0) = ',num2str(x(1)),])
disp('         t             u*(t)       x*(t)   ')
disp('         _________________________________')
disp([t(2:end), u', x(2:end)'])

disp(['         J10 = ',num2str(sum),])

format

close all
subplot(2,3,2)
plot(t(1),xtar','gh')
hold on
plot(t,x,'r*-')
hold on
plot(t(1),x0,'ks')
hold on
legend('x\_{tar}','x*_t','x_0')
title('State');
plot(t,xtar*ones(size(t)),'g-.')
axis([min(t) max(t) min(x)-10 xtar+10]);
hold off

subplot(2,3,5)
plot(0,utar','c^')
hold on
plot(t,[NaN,u],'bp-')
hold on
legend('u\_tar','u*_{t+1}')
title('Input');
xlabel('t')
plot(t,utar*ones(size(t)),'c-.')
axis([min(t) max(t) utar-5 max(u)+5]);

